' ridge regression  (following Neter, Kutner, et al, p. 411-413)
' on input:
' 	y - dependent variable
' 	x - group consisting of independent variables (including C if desired)
' 	lambda - ridge adjustment
'      smplser -  a series equal to 1 at every observation you want to use.
' on output:
'	b - coefficient estimate
' 	bridge - standardized coefficient estimate
'	vif - variance inflation factors
'      r2 - r-squared
subroutine local ridgereg(vector b, vector bridge, vector vif, scalar !r2, series y, group g, scalar lambda, series smplser)
	local smpl	
	' drop missing observations
	smpl if y<>na and @rnas(g)=0 and smplser=1
	series temp = 1
	smplser = temp
	!obs = @obs(y)
	!obsm1 = !obs - 1
	
	' get a modified version of the original group after dropping the constnat
	group tempx g
	tempx.drop c

	setmaxerrs 2
	' compute the ridge regression and standardized ridge regression coefficients
	group tempyx y tempx
	sym ss = @cov(tempyx)*!obs ' sum-of-deviations from mean
	sym xx = @subextract(ss, 2, 2)  ' centered x'x
	matrix xy = @subextract(ss, 2, 1, @rows(ss), 1) ' centered x'y
	vector dscale = @getmaindiagonal(xx) ' diagonal scale
	sym dd = @makediagonal(@epow(dscale,  -.5) ) ' diagonal scale weighting matrix
	
	vector btemp =  dd * @inverse(dd*xx*dd+ lambda*@identity(@columns(xx))) * dd *  xy  ' get ridge regression coefs
	if @lasterrnum>0 then
		return
	endif
	vector bridge = @makediagonal(@epow(dscale, .5)) * btemp / @sqrt(ss(1)) ' get standardized ridge regression coefficients
	matrix pmat = @inverse(dd*xx*dd + lambda*@identity(@columns(xx))) * dd*xx*dd * @inverse(dd*xx*dd + lambda*@identity(@columns(xx)))
	vif = @getmaindiagonal(pmat)
	
	' final adjustments to handle c coefficient and to export coefficients in proper order
	group tempx2 c tempx
	%names = g.@members
	g.drop {%names}
	g.add tempx2
	vector(g.@count) b
	!j = 1
	!cval = @mean(y)
	for !i=2 to g.@count
			b(!i) = btemp(!j)
			%name = g.@seriesname(!i)
			!cval = !cval - @mean({%name})*b(!i)
			!j = !j + 1
	next
	b(1) = !cval
	
	'calculate r-squared
	call r2(!r2,b,y,g)	
endsub


subroutine local r2(scalar !R2, vector coefs, series y, group g)
	!meany = @mean(y)
	!sstot = @sumsq(y -!meany)
	series residuals = y 
	for !j=1 to g.@count
		residuals = residuals - coefs(!j)*g(!j)
	next
	!ssr = @sumsq(residuals)
	!r2 = 1-!ssr/!sstot
endsub
