include .\ridgelib.prg

%sample = @pagesmpl
%origsmpl = %sample

%dep = ""
%regs = ""
%lambda = "0 0.02 0.04 0.06 0.08 0.1 0.2 0.5 1"
!result = @uidialog("caption", "Ridge Regression Trace", _
		 "edit", %dep, "Enter a dependent variable", _
		 "edit", %regs, "Enter a list of regressors (a constant will be automatically added)", 250, _
		 "edit", %lambda, "Enter a space delimited list of values, or the name of a vector object containing values, for the biasing constant", 250, _
		 "edit", %sample, "Sample")
if !result = -1 then
	stop
endif

'check valid series name for dep
if @len(%dep) = 0 then
	@uiprompt("Please enter a series name for the dependent variable")
	stop
endif
%grname = @getnextname("_gr")
group {%grname} {%dep}
if @lasterrnum>0 then
	@uiprompt("Invalid dependent variable")	
	stop
endif
d {%grname}
%depv = @getnextname("_depv")
series {%depv} = {%dep}

'check valid regressors
if @len(%regs) = 0 then
	@uiprompt("Please enter a list of regressors")
	stop
endif
setmaxerrs 2
%gname = @getnextname("_g")
group {%gname} {%regs}
if @lasterrnum>0 then
	d(noerr) {%gname}
	@uiprompt("Invalid list of regressors")
	stop
endif

'check sample
smpl {%sample}
if @lasterrnum>0 then
	d(noerr) {%gname}
	@uiprompt("Invalid sample")
	smpl {%origsmpl}
	stop
endif

%vname = @getnextname("_vec")
!isvec=0
'check valid value for lambda
if @wcount(%lambda) = 1 then
	if @isobject(%lambda) = 1 then
		if {%lambda}.@detailedtype="VECTOR" then
			vector {%vname} = {%lambda}
			!isvec = 1
		endif
	endif
endif

if !isvec=0 then
	vector(@wcount(%lambda)) {%vname}
	for !i=1 to @wcount(%lambda)
		%l = @word(%lambda,!i)
		!l = @val(%l)
		if !l = NA then
			%msg = %l + " is an invalid value for the biasing constant"
			@uiprompt(%msg)
			d(noerr) {%gname}
			stop
		endif
		{%vname}(!i) = !l
	next
endif

%sname = "output"
if @isobject(%sname) then
	%sname = @getnextname(%sname)
endif
spool {%sname}

call ridgetrtab({%sname}, {%depv}, {%gname}, {%vname}, %sample)
show {%sname}
d(noerr) {%gname} {%vname} {%depv}

subroutine local ridgetrtab(spool output, series y, group x, vector vec, string %smplstr)
	!hasc = @wfindnc(x.@members,"C")>0
	matrix(@rows(vec), x.@count-!hasc) stcoefs
	matrix(@rows(vec), x.@count-!hasc) vifs
	vector bcoef
	vector bridge
	!R2=0
	vector vif
	series smplser = 1
	'loop through values of lambda
	for !i=1 to @rows(vec)
		!l = vec(!i)
		call ridgereg(bcoef, bridge, vif, !r2, y, x, !l, smplser)
		rowplace(stcoefs,@transpose(bridge),!i)
		rowplace(vifs,@transpose(vif),!i)
	next
	
	matrix(@rows(vec),@columns(stcoefs)+1) bigcoefs
	colplace(bigcoefs,vec,1)
	matplace(bigcoefs,stcoefs,1,2)
	freeze(coefgraph) bigcoefs.xyline
	for !I=1 to x.@count
		%n = x.@seriesname(!i)
		coefgraph.name(!i+1) {%n}
	next
	coefgraph.name(1) Biasing Constant	
	
	output.append coefgraph
	output.displayname 1 "Ridge Trace"
	output.options displaynames
	
	table tab
	
	tab(2,1) = "Biasing"
	tab(2,3) = "V.I.F.s"
	tab(3,1) = "Constant"
	for !i=2 to x.@count
		tab(3,!i) = x.@seriesname(!i)
	next
	tab.setlines(4) +d
	for !i=1 to x.@count-1
		for !j=1 to @rows(vec)
			tab(!j+4,1) = vec(!j)
			tab(!j+4,!i+1) = vifs(!j,!i)
		next
	next
	tab.setlines(1) +d
	!end = 5+@rows(vec)
	tab.setlines(!end) +d
	output.append tab
	output.displayname 2 "V.I.F.s"
	
endsub
